@@ -29,11 +29,11 @@ public:
   * \param boosting Input boosting model
   * \param num_iteration Number of boosting round
   * \param is_raw_score True if need to predict result with raw score
-  * \param is_predict_leaf_index True to output leaf index instead of prediction score
-  * \param is_predict_contrib True to output feature contributions instead of prediction score
+  * \param predict_leaf_index True to output leaf index instead of prediction score
+  * \param predict_contrib True to output feature contributions instead of prediction score
   */
   Predictor(Boosting* boosting, int num_iteration,
-            bool is_raw_score, bool is_predict_leaf_index, bool is_predict_contrib,
+            bool is_raw_score, bool predict_leaf_index, bool predict_contrib,
             bool early_stop, int early_stop_freq, double early_stop_margin) {
 
     early_stop_ = CreatePredictionEarlyStopInstance("none", LightGBM::PredictionEarlyStopConfig());
@@ -55,14 +55,14 @@ public:
     {
       num_threads_ = omp_get_num_threads();
     }
-    boosting->InitPredict(num_iteration, is_predict_contrib);
+    boosting->InitPredict(num_iteration, predict_contrib);
     boosting_ = boosting;
-    num_pred_one_row_ = boosting_->NumPredictOneRow(num_iteration, is_predict_leaf_index, is_predict_contrib);
+    num_pred_one_row_ = boosting_->NumPredictOneRow(num_iteration, predict_leaf_index, predict_contrib);
     num_feature_ = boosting_->MaxFeatureIdx() + 1;
     predict_buf_ = std::vector<std::vector<double>>(num_threads_, std::vector<double>(num_feature_, 0.0f));
-    const int kFeatureThreshold = 20000;
-    const size_t KSparseThreshold = static_cast<size_t>(0.02 * num_feature_);
-    if (is_predict_leaf_index) {
+    const int kFeatureThreshold = 100000;
+    const size_t KSparseThreshold = static_cast<size_t>(0.01 * num_feature_);
+    if (predict_leaf_index) {
       predict_fun_ = [this, kFeatureThreshold, KSparseThreshold](const std::vector<std::pair<int, double>>& features, double* output) {
         int tid = omp_get_thread_num();
         if (num_feature_ > kFeatureThreshold && features.size() < KSparseThreshold) {
@@ -75,7 +75,7 @@ public:
           ClearPredictBuffer(predict_buf_[tid].data(), predict_buf_[tid].size(), features);
         }
       };
-    } else if (is_predict_contrib) {
+    } else if (predict_contrib) {
       predict_fun_ = [this](const std::vector<std::pair<int, double>>& features, double* output) {
         int tid = omp_get_thread_num();
         CopyToPredictBuffer(predict_buf_[tid].data(), features);
@@ -127,34 +127,27 @@ public:
   * \param data_filename Filename of data
   * \param result_filename Filename of output result
   */
-  void Predict(const char* data_filename, const char* result_filename, bool has_header) {
-    FILE* result_file;
-
-    #ifdef _MSC_VER
-    fopen_s(&result_file, result_filename, "w");
-    #else
-    result_file = fopen(result_filename, "w");
-    #endif
-
-    if (result_file == NULL) {
-      Log::Fatal("Prediction results file %s cannot be found.", result_filename);
+  void Predict(const char* data_filename, const char* result_filename, bool header) {
+    auto writer = VirtualFileWriter::Make(result_filename);
+    if (!writer->Init()) {
+      Log::Fatal("Prediction results file %s cannot be found", result_filename);
     }
-    auto parser = std::unique_ptr<Parser>(Parser::CreateParser(data_filename, has_header, boosting_->MaxFeatureIdx() + 1, boosting_->LabelIdx()));
+    auto parser = std::unique_ptr<Parser>(Parser::CreateParser(data_filename, header, boosting_->MaxFeatureIdx() + 1, boosting_->LabelIdx()));
 
     if (parser == nullptr) {
-      Log::Fatal("Could not recognize the data format of data file %s.", data_filename);
+      Log::Fatal("Could not recognize the data format of data file %s", data_filename);
     }
 
-    TextReader<data_size_t> predict_data_reader(data_filename, has_header);
+    TextReader<data_size_t> predict_data_reader(data_filename, header);
     std::unordered_map<int, int> feature_names_map_;
     bool need_adjust = false;
-    if (has_header) {
+    if (header) {
       std::string first_line = predict_data_reader.first_line();
-      std::vector<std::string> header = Common::Split(first_line.c_str(), "\t,");
-      header.erase(header.begin() + boosting_->LabelIdx());
-      for (int i = 0; i < static_cast<int>(header.size()); ++i) {
+      std::vector<std::string> header_words = Common::Split(first_line.c_str(), "\t,");
+      header_words.erase(header_words.begin() + boosting_->LabelIdx());
+      for (int i = 0; i < static_cast<int>(header_words.size()); ++i) {
         for (int j = 0; j < static_cast<int>(boosting_->FeatureNames().size()); ++j) {
-          if (header[i] == boosting_->FeatureNames()[j]) {
+          if (header_words[i] == boosting_->FeatureNames()[j]) {
             feature_names_map_[i] = j;
             break;
           }
@@ -189,7 +182,7 @@ public:
     };
 
     std::function<void(data_size_t, const std::vector<std::string>&)> process_fun =
-      [this, &parser_fun, &result_file]
+      [this, &parser_fun, &writer]
     (data_size_t, const std::vector<std::string>& lines) {
       std::vector<std::pair<int, double>> oneline_features;
       std::vector<std::string> result_to_write(lines.size());
@@ -209,11 +202,11 @@ public:
       }
       OMP_THROW_EX();
       for (data_size_t i = 0; i < static_cast<data_size_t>(result_to_write.size()); ++i) {
-        fprintf(result_file, "%s\n", result_to_write[i].c_str());
+        writer->Write(result_to_write[i].c_str(), result_to_write[i].size());
+        writer->Write("\n", 1);
       }
     };
     predict_data_reader.ReadAllAndProcessParallel(process_fun);
-    fclose(result_file);
   }
 
 private:
