@@ -214,7 +214,7 @@ void Dataset::Construct(
   int** sample_non_zero_indices,
   const int* num_per_col,
   size_t total_sample_cnt,
-  const IOConfig& io_config) {
+  const Config& io_config) {
 
   num_total_features_ = static_cast<int>(bin_mappers.size());
   sparse_threshold_ = io_config.sparse_threshold;
@@ -226,10 +226,10 @@ void Dataset::Construct(
     }
   }
   if (used_features.empty()) {
-    Log::Fatal("Cannot construct Dataset since there are not useful features. \
-                It should be at least two unique rows. \
-                If the num_row (num_data) is small, you can set min_data=1 and min_data_in_bin=1 to fix this. \
-                Otherwise please make sure you are using the right dataset.");
+    Log::Fatal("Cannot construct Dataset since there are not useful features.\n"
+               "It should be at least two unique rows.\n"
+               "If the num_row (num_data) is small, you can set min_data=1 and min_data_in_bin=1 to fix this.\n"
+               "Otherwise please make sure you are using the right dataset");
   }
   auto features_in_group = NoGroup(used_features);
 
@@ -292,6 +292,33 @@ void Dataset::Construct(
       last_group = group;
     }
   }
+
+  if (!io_config.monotone_constraints.empty()) {
+    CHECK(static_cast<size_t>(num_total_features_) == io_config.monotone_constraints.size());
+    monotone_types_.resize(num_features_);
+    for (int i = 0; i < num_total_features_; ++i) {
+      int inner_fidx = InnerFeatureIndex(i);
+      if (inner_fidx >= 0) {
+        monotone_types_[inner_fidx] = io_config.monotone_constraints[i];
+      }
+    }
+    if (ArrayArgs<int8_t>::CheckAllZero(monotone_types_)) {
+      monotone_types_.clear();
+    }
+  }
+  if (!io_config.feature_contri.empty()) {
+    CHECK(static_cast<size_t>(num_total_features_) == io_config.feature_contri.size());
+    feature_penalty_.resize(num_features_);
+    for (int i = 0; i < num_total_features_; ++i) {
+      int inner_fidx = InnerFeatureIndex(i);
+      if (inner_fidx >= 0) {
+        feature_penalty_[inner_fidx] = std::max(0.0, io_config.feature_contri[i]);
+      }
+    }
+    if (ArrayArgs<double>::CheckAll(feature_penalty_, 1.0)) {
+      feature_penalty_.clear();
+    }
+  }
 }
 
 void Dataset::FinishLoad() {
@@ -335,6 +362,8 @@ void Dataset::CopyFeatureMapperFrom(const Dataset* dataset) {
   group_bin_boundaries_ = dataset->group_bin_boundaries_;
   group_feature_start_ = dataset->group_feature_start_;
   group_feature_cnt_ = dataset->group_feature_cnt_;
+  monotone_types_ = dataset->monotone_types_;
+  feature_penalty_ = dataset->feature_penalty_;
 }
 
 void Dataset::CreateValid(const Dataset* dataset) {
@@ -387,6 +416,8 @@ void Dataset::CreateValid(const Dataset* dataset) {
       last_group = group;
     }
   }
+  monotone_types_ = dataset->monotone_types_;
+  feature_penalty_ = dataset->feature_penalty_;
 }
 
 void Dataset::ReSize(data_size_t num_data) {
@@ -424,13 +455,13 @@ bool Dataset::SetFloatField(const char* field_name, const float* field_data, dat
   name = Common::Trim(name);
   if (name == std::string("label") || name == std::string("target")) {
     #ifdef LABEL_T_USE_DOUBLE
-    Log::Fatal("Don't Support LABEL_T_USE_DOUBLE.");
+    Log::Fatal("Don't support LABEL_T_USE_DOUBLE");
     #else
     metadata_.SetLabel(field_data, num_element);
     #endif
   } else if (name == std::string("weight") || name == std::string("weights")) {
     #ifdef LABEL_T_USE_DOUBLE
-    Log::Fatal("Don't Support LABEL_T_USE_DOUBLE.");
+    Log::Fatal("Don't support LABEL_T_USE_DOUBLE");
     #else
     metadata_.SetWeights(field_data, num_element);
     #endif
@@ -467,14 +498,14 @@ bool Dataset::GetFloatField(const char* field_name, data_size_t* out_len, const
   name = Common::Trim(name);
   if (name == std::string("label") || name == std::string("target")) {
     #ifdef LABEL_T_USE_DOUBLE
-    Log::Fatal("Don't Support LABEL_T_USE_DOUBLE.");
+    Log::Fatal("Don't support LABEL_T_USE_DOUBLE");
     #else
     *out_ptr = metadata_.label();
     *out_len = num_data_;
     #endif
   } else if (name == std::string("weight") || name == std::string("weights")) {
     #ifdef LABEL_T_USE_DOUBLE
-    Log::Fatal("Don't Support LABEL_T_USE_DOUBLE.");
+    Log::Fatal("Don't support LABEL_T_USE_DOUBLE");
     #else
     *out_ptr = metadata_.weights();
     *out_len = num_data_;
@@ -512,7 +543,7 @@ bool Dataset::GetIntField(const char* field_name, data_size_t* out_len, const in
 void Dataset::SaveBinaryFile(const char* bin_filename) {
   if (bin_filename != nullptr
       && std::string(bin_filename) == data_filename_) {
-    Log::Warning("Bianry file %s already existed", bin_filename);
+    Log::Warning("Bianry file %s already exists", bin_filename);
     return;
   }
   // if not pass a filename, just append ".bin" of original file
@@ -522,77 +553,79 @@ void Dataset::SaveBinaryFile(const char* bin_filename) {
     bin_filename = bin_filename_str.c_str();
   }
   bool is_file_existed = false;
-  FILE* file;
-  #ifdef _MSC_VER
-  fopen_s(&file, bin_filename, "rb");
-  #else
-  file = fopen(bin_filename, "rb");
-  #endif
 
-  if (file != NULL) {
+  if (VirtualFileWriter::Exists(bin_filename)) {
     is_file_existed = true;
-    Log::Warning("File %s existed, cannot save binary to it", bin_filename);
-    fclose(file);
+    Log::Warning("File %s exists, cannot save binary to it", bin_filename);
   }
 
   if (!is_file_existed) {
-    #ifdef _MSC_VER
-    fopen_s(&file, bin_filename, "wb");
-    #else
-    file = fopen(bin_filename, "wb");
-    #endif
-    if (file == NULL) {
+    auto writer = VirtualFileWriter::Make(bin_filename);
+    if (!writer->Init()) {
       Log::Fatal("Cannot write binary data to %s ", bin_filename);
     }
     Log::Info("Saving data to binary file %s", bin_filename);
     size_t size_of_token = std::strlen(binary_file_token);
-    fwrite(binary_file_token, sizeof(char), size_of_token, file);
+    writer->Write(binary_file_token, size_of_token);
     // get size of header
     size_t size_of_header = sizeof(num_data_) + sizeof(num_features_) + sizeof(num_total_features_)
       + sizeof(int) * num_total_features_ + sizeof(label_idx_) + sizeof(num_groups_)
-      + 3 * sizeof(int) * num_features_ + sizeof(uint64_t) * (num_groups_ + 1) + 2 * sizeof(int) * num_groups_;
+      + 3 * sizeof(int) * num_features_ + sizeof(uint64_t) * (num_groups_ + 1) + 2 * sizeof(int) * num_groups_ + sizeof(int8_t) * num_features_
+      + sizeof(double) * num_features_;
     // size of feature names
     for (int i = 0; i < num_total_features_; ++i) {
       size_of_header += feature_names_[i].size() + sizeof(int);
     }
-    fwrite(&size_of_header, sizeof(size_of_header), 1, file);
+    writer->Write(&size_of_header, sizeof(size_of_header));
     // write header
-    fwrite(&num_data_, sizeof(num_data_), 1, file);
-    fwrite(&num_features_, sizeof(num_features_), 1, file);
-    fwrite(&num_total_features_, sizeof(num_total_features_), 1, file);
-    fwrite(&label_idx_, sizeof(label_idx_), 1, file);
-    fwrite(used_feature_map_.data(), sizeof(int), num_total_features_, file);
-    fwrite(&num_groups_, sizeof(num_groups_), 1, file);
-    fwrite(real_feature_idx_.data(), sizeof(int), num_features_, file);
-    fwrite(feature2group_.data(), sizeof(int), num_features_, file);
-    fwrite(feature2subfeature_.data(), sizeof(int), num_features_, file);
-    fwrite(group_bin_boundaries_.data(), sizeof(uint64_t), num_groups_ + 1, file);
-    fwrite(group_feature_start_.data(), sizeof(int), num_groups_, file);
-    fwrite(group_feature_cnt_.data(), sizeof(int), num_groups_, file);
-
+    writer->Write(&num_data_, sizeof(num_data_));
+    writer->Write(&num_features_, sizeof(num_features_));
+    writer->Write(&num_total_features_, sizeof(num_total_features_));
+    writer->Write(&label_idx_, sizeof(label_idx_));
+    writer->Write(used_feature_map_.data(), sizeof(int) * num_total_features_);
+    writer->Write(&num_groups_, sizeof(num_groups_));
+    writer->Write(real_feature_idx_.data(), sizeof(int) * num_features_);
+    writer->Write(feature2group_.data(), sizeof(int) * num_features_);
+    writer->Write(feature2subfeature_.data(), sizeof(int) * num_features_);
+    writer->Write(group_bin_boundaries_.data(), sizeof(uint64_t) * (num_groups_ + 1));
+    writer->Write(group_feature_start_.data(), sizeof(int) * num_groups_);
+    writer->Write(group_feature_cnt_.data(), sizeof(int) * num_groups_);
+    if (monotone_types_.empty()) {
+      ArrayArgs<int8_t>::Assign(&monotone_types_, 0, num_features_);
+    }
+    writer->Write(monotone_types_.data(), sizeof(int8_t) * num_features_);
+    if (ArrayArgs<int8_t>::CheckAllZero(monotone_types_)) {
+      monotone_types_.clear();
+    }
+    if (feature_penalty_.empty()) {
+      ArrayArgs<double>::Assign(&feature_penalty_, 1.0, num_features_);
+    }
+    writer->Write(feature_penalty_.data(), sizeof(double) * num_features_);
+    if (ArrayArgs<double>::CheckAll(feature_penalty_, 1.0)) {
+      feature_penalty_.clear();
+    }
     // write feature names
     for (int i = 0; i < num_total_features_; ++i) {
       int str_len = static_cast<int>(feature_names_[i].size());
-      fwrite(&str_len, sizeof(int), 1, file);
+      writer->Write(&str_len, sizeof(int));
       const char* c_str = feature_names_[i].c_str();
-      fwrite(c_str, sizeof(char), str_len, file);
+      writer->Write(c_str, sizeof(char) * str_len);
     }
 
     // get size of meta data
     size_t size_of_metadata = metadata_.SizesInByte();
-    fwrite(&size_of_metadata, sizeof(size_of_metadata), 1, file);
+    writer->Write(&size_of_metadata, sizeof(size_of_metadata));
     // write meta data
-    metadata_.SaveBinaryToFile(file);
+    metadata_.SaveBinaryToFile(writer.get());
 
     // write feature data
     for (int i = 0; i < num_groups_; ++i) {
       // get size of feature
       size_t size_of_feature = feature_groups_[i]->SizesInByte();
-      fwrite(&size_of_feature, sizeof(size_of_feature), 1, file);
+      writer->Write(&size_of_feature, sizeof(size_of_feature));
       // write feature
-      feature_groups_[i]->SaveBinaryToFile(file);
+      feature_groups_[i]->SaveBinaryToFile(writer.get());
     }
-    fclose(file);
   }
 }
 
@@ -613,13 +646,17 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
   used_group.reserve(num_groups_);
   for (int group = 0; group < num_groups_; ++group) {
     const int f_cnt = group_feature_cnt_[group];
+    bool is_group_used = false;
     for (int j = 0; j < f_cnt; ++j) {
       const int fidx = group_feature_start_[group] + j;
       if (is_feature_used[fidx]) {
+        is_group_used = true;
         break;
       }
     }
-    used_group.push_back(group);
+    if (is_group_used) {
+      used_group.push_back(group);
+    }
   }
   int num_used_group = static_cast<int>(used_group.size());
   auto ptr_ordered_grad = gradients;
@@ -648,7 +685,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset(data_ptr + 1, 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
         if (ordered_bins[group] == nullptr) {
           // if not use ordered bin
@@ -677,7 +714,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset(data_ptr + 1, 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
         if (ordered_bins[group] == nullptr) {
           // if not use ordered bin
@@ -710,7 +747,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset(data_ptr + 1, 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
         if (ordered_bins[group] == nullptr) {
           // if not use ordered bin
@@ -738,7 +775,7 @@ void Dataset::ConstructHistograms(const std::vector<int8_t>& is_feature_used,
         // feature is not used
         auto data_ptr = hist_data + group_bin_boundaries_[group];
         const int num_bin = feature_groups_[group]->num_total_bin_;
-        std::memset(data_ptr + 1, 0, (num_bin - 1) * sizeof(HistogramBinEntry));
+        std::memset((void*)(data_ptr + 1), 0, (num_bin - 1) * sizeof(HistogramBinEntry));
         // construct histograms for smaller leaf
         if (ordered_bins[group] == nullptr) {
           // if not use ordered bin
