@@ -308,7 +308,7 @@ void Metadata::SetLabel(const label_t* label, data_size_t len) {
     Log::Fatal("label cannot be nullptr");
   }
   if (num_data_ != len) {
-    Log::Fatal("len of label is not same with #data");
+    Log::Fatal("Length of label is not same with #data");
   }
   if (!label_.empty()) { label_.clear(); }
   label_ = std::vector<label_t>(num_data_);
@@ -327,7 +327,7 @@ void Metadata::SetWeights(const label_t* weights, data_size_t len) {
     return;
   }
   if (num_data_ != len) {
-    Log::Fatal("len of weights is not same with #data");
+    Log::Fatal("Length of weights is not same with #data");
   }
   if (!weights_.empty()) { weights_.clear(); }
   num_weights_ = num_data_;
@@ -354,7 +354,7 @@ void Metadata::SetQuery(const data_size_t* query, data_size_t len) {
     sum += query[i];
   }
   if (num_data_ != sum) {
-    Log::Fatal("sum of query counts is not same with #data");
+    Log::Fatal("Sum of query counts is not same with #data");
   }
   if (!query_boundaries_.empty()) { query_boundaries_.clear(); }
   num_queries_ = len;
@@ -424,7 +424,7 @@ void Metadata::LoadInitialScore(const char* initscore_file) {
       double tmp = 0.0f;
       oneline_init_score = Common::Split(reader.Lines()[i].c_str(), '\t');
       if (static_cast<int>(oneline_init_score.size()) != num_class) {
-        Log::Fatal("Invalid initial score file. Redundant or insufficient columns.");
+        Log::Fatal("Invalid initial score file. Redundant or insufficient columns");
       }
       for (int k = 0; k < num_class; ++k) {
         Common::Atof(oneline_init_score[k].c_str(), &tmp);
@@ -505,16 +505,16 @@ void Metadata::LoadFromMemory(const void* memory) {
   LoadQueryWeights();
 }
 
-void Metadata::SaveBinaryToFile(FILE* file) const {
-  fwrite(&num_data_, sizeof(num_data_), 1, file);
-  fwrite(&num_weights_, sizeof(num_weights_), 1, file);
-  fwrite(&num_queries_, sizeof(num_queries_), 1, file);
-  fwrite(label_.data(), sizeof(label_t), num_data_, file);
+void Metadata::SaveBinaryToFile(const VirtualFileWriter* writer) const {
+  writer->Write(&num_data_, sizeof(num_data_));
+  writer->Write(&num_weights_, sizeof(num_weights_));
+  writer->Write(&num_queries_, sizeof(num_queries_));
+  writer->Write(label_.data(), sizeof(label_t) * num_data_);
   if (!weights_.empty()) {
-    fwrite(weights_.data(), sizeof(label_t), num_weights_, file);
+    writer->Write(weights_.data(), sizeof(label_t) * num_weights_);
   }
   if (!query_boundaries_.empty()) {
-    fwrite(query_boundaries_.data(), sizeof(data_size_t), num_queries_ + 1, file);
+    writer->Write(query_boundaries_.data(), sizeof(data_size_t) * (num_queries_ + 1));
   }
 
 }
