@@ -20,6 +20,9 @@ if is_py3:
     def argc_(func):
         """return number of arguments of a function"""
         return len(inspect.signature(func).parameters)
+
+    def decode_string(bytestring):
+        return bytestring.decode('utf-8')
 else:
     string_type = basestring
     numeric_types = (int, long, float, bool)
@@ -30,6 +33,9 @@ else:
         """return number of arguments of a function"""
         return len(inspect.getargspec(func).args)
 
+    def decode_string(bytestring):
+        return bytestring
+
 """json"""
 try:
     import simplejson as json
@@ -51,18 +57,36 @@ def json_default_with_numpy(obj):
 """pandas"""
 try:
     from pandas import Series, DataFrame
+    PANDAS_INSTALLED = True
 except ImportError:
+    PANDAS_INSTALLED = False
+
     class Series(object):
         pass
 
     class DataFrame(object):
         pass
 
+"""matplotlib"""
+try:
+    import matplotlib
+    MATPLOTLIB_INSTALLED = True
+except ImportError:
+    MATPLOTLIB_INSTALLED = False
+
+"""graphviz"""
+try:
+    import graphviz
+    GRAPHVIZ_INSTALLED = True
+except ImportError:
+    GRAPHVIZ_INSTALLED = False
+
 """sklearn"""
 try:
     from sklearn.base import BaseEstimator
     from sklearn.base import RegressorMixin, ClassifierMixin
     from sklearn.preprocessing import LabelEncoder
+    from sklearn.utils.class_weight import compute_sample_weight
     from sklearn.utils.multiclass import check_classification_targets
     from sklearn.utils.validation import check_X_y, check_array, check_consistent_length
     try:
@@ -83,6 +107,7 @@ try:
     _LGBMCheckArray = check_array
     _LGBMCheckConsistentLength = check_consistent_length
     _LGBMCheckClassificationTargets = check_classification_targets
+    _LGBMComputeSampleWeight = compute_sample_weight
 except ImportError:
     SKLEARN_INSTALLED = False
     _LGBMModelBase = object
@@ -96,6 +121,7 @@ except ImportError:
     _LGBMCheckArray = None
     _LGBMCheckConsistentLength = None
     _LGBMCheckClassificationTargets = None
+    _LGBMComputeSampleWeight = None
 
 
 # DeprecationWarning is not shown by default, so let's create our own with higher level
